//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#define SendNextStatement_SUCCESS 1
#define SendNextStatement_FAILURE 2
#define SendNextStatement_DONE    3

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int SendNextStatement(int Client)
{
    SQLINTEGER StatementLen = 0;
    SQLCHAR Statement[1024 + 1];
    SQLINTEGER ImportTableLen = 0;
    SQLCHAR ImportTable[32 + 1];
    char StatusText[255];

    char SendData[MaxReceiveSize + 1];

    if(!CI[Client].DBClass.DBFetch(CI[Client].StatementHandle))
    {
        // This is not an error. It means there are no more rows.
        CI[Client].DBClass.DBCloseCursor(CI[Client].StatementHandle);
        return SendNextStatement_DONE;
    }

    if(!CI[Client].DBClass.DBGetData(CI[Client].StatementHandle, 1, SQL_C_CHAR, Statement, 1024, &StatementLen))
    {
        sprintf(StatusText, "(%d) DBGetData Statement error.", MyServer.ClientID[Client]);
        WriteLog(StatusText);

        CI[Client].DBClass.DBCloseCursor(CI[Client].StatementHandle);
        return SendNextStatement_FAILURE;
    }
    if(StatementLen == -1)
	{
	    CI[Client].DBClass.DBCloseCursor(CI[Client].StatementHandle);
		return SendNextStatement_FAILURE;
	}

    if(!CI[Client].DBClass.DBGetData(CI[Client].StatementHandle, 2, SQL_C_CHAR, ImportTable, 32, &ImportTableLen))
    {
        sprintf(StatusText, "(%d) DBGetData ImportTable error.", MyServer.ClientID[Client]);
        WriteLog(StatusText);
        CI[Client].DBClass.DBCloseCursor(CI[Client].StatementHandle);
        return SendNextStatement_FAILURE;
    }
    if(StatementLen == -1)
	{
	    CI[Client].DBClass.DBCloseCursor(CI[Client].StatementHandle);
		return SendNextStatement_FAILURE;
	}

    sprintf(SendData, "::ImportTable:%s", ImportTable);
    SimpleServerSend(Client, SendData);

    sprintf(SendData, "::Statement:%s", Statement);
    SimpleServerSend(Client, SendData);

    return SendNextStatement_SUCCESS;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int ProcessCommand(int Client, char *ReceivedData, int ReceivedSize)
{
    int FlagLength = 0;
    int FlaggedLength = 0;
    char FlaggedData[MaxReceiveSize + 1];
    char StatusTemp[255];

    if(strcmp(ReceivedData, "::RequestFirstStatement") == 0)
    {
        sprintf(StatusTemp, "(%d) Received Command: RequestFirstStatement.", MyServer.ClientID[Client]);
        WriteLog(StatusTemp);

        if(!CI[Client].DBClass.DBConnect("{SQL Server}", "NetBase", "sa", "SQL840823110", CI[Client].CompanyDB))
        {
            sprintf(StatusTemp, "(%d) Command: Error connecting to the web portal customer database.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);
            return COMMAND_RESULT_ERROR;
        }

        char Statement[255];
        sprintf(Statement, "SELECT Statement, ImportTable FROM WP_Statements WHERE Active = 1 ORDER BY Sequence");

        if(!CI[Client].DBClass.DBExecute(Statement, &CI[Client].StatementHandle))
        {
            sprintf(StatusTemp, "(%d) Command: DBexecute filed in RequestFirstStatement.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);
            return COMMAND_RESULT_ERROR;
        }
        else{
            sprintf(StatusTemp, "(%d) Command: RequestFirstStatement DBexecute success.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);
        }

        int SendResult = SendNextStatement(Client);
        if(SendResult == SendNextStatement_SUCCESS){
            return COMMAND_RESULT_OK;
        }
        else if(SendResult == SendNextStatement_FAILURE){
            sprintf(StatusTemp, "(%d) Command: Failed to send first statement.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);
            SimpleServerSend(Client, "::Error:Server failed to send first statement.");
            return COMMAND_RESULT_ERROR;
        }
        else if(SendResult == SendNextStatement_DONE){
            sprintf(StatusTemp, "(%d) Command: End of statements.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);
            SimpleServerSend(Client, "::EndOfStatements");
            return COMMAND_RESULT_OK;
        }

        sprintf(StatusTemp, "(%d) Error in ProcessCommand :: Logic failed in RequestFirstStatement.", MyServer.ClientID[Client]);
        WriteLog(StatusTemp);

        return COMMAND_RESULT_ERROR; // Should never get here
    }
    else if(strcmp(ReceivedData, "::RequestNextStatement") == 0)
    {
        sprintf(StatusTemp, "(%d) Received Command: RequestNextStatement.", MyServer.ClientID[Client]);
        WriteLog(StatusTemp);

        int SendResult = SendNextStatement(Client);
        if(SendResult == SendNextStatement_SUCCESS){
            return COMMAND_RESULT_OK;
        }
        else if(SendResult == SendNextStatement_FAILURE){
            sprintf(StatusTemp, "(%d) Command: Failed to send next statement.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);
            SimpleServerSend(Client, "::Error:Server failed to send next statement.");
            return COMMAND_RESULT_ERROR;
        }
        else if(SendResult == SendNextStatement_DONE){
            sprintf(StatusTemp, "(%d) Command: End of statements.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);
            SimpleServerSend(Client, "::EndOfStatements");
            return COMMAND_RESULT_OK;
        }

        return COMMAND_RESULT_ERROR; // Should never get here
    }
    else if((FlagLength = CompareFlagedString(ReceivedData, "::FileName:")))
    {
        sprintf(StatusTemp, "(%d) Received Command: FileName.", MyServer.ClientID[Client]);
        WriteLog(StatusTemp);

    	FlaggedLength = GetFlagedValue(ReceivedData, FlagLength, ReceivedSize, FlaggedData);
        FlaggedData[FlaggedLength] = '\0';

        sprintf(CI[Client].FileName, "%s\\%s\\%s.wpd", DataDirectory, CI[Client].CompanyName, FlaggedData);
        CreateFolderStructure(CI[Client].FileName);

        sprintf(StatusTemp, "(%d) Command: FileName: '%s'.", MyServer.ClientID[Client], CI[Client].FileName);
        WriteLog(StatusTemp);

        if(ReceiveFileData(Client, CI[Client].FileName))
        {
            SimpleServerSend(Client, "::FileCRC");
            return COMMAND_RESULT_OK;
        }

        return COMMAND_RESULT_ERROR;
    }
    else if((FlagLength = CompareFlagedString(ReceivedData, "::FileCRC:")))
    {
        sprintf(StatusTemp, "(%d) Received Command: FileCRC.", MyServer.ClientID[Client]);
        WriteLog(StatusTemp);

    	FlaggedLength = GetFlagedValue(ReceivedData, FlagLength, ReceivedSize, FlaggedData);
        FlaggedData[FlaggedLength] = '\0';

        DWORD ClientFileCRC = atol(FlaggedData);

        if(ClientFileCRC == CI[Client].DataCRC)
        {
            sprintf(StatusTemp, "(%d) Command: Data transfer CRC success.", MyServer.ClientID[Client]);
            WriteLog(StatusTemp);

            if(ImportSQLResults(Client, CI[Client].FileName))
            {
                SimpleServerSend(Client, "::LastImportSuccess");
                // FIXME: Should we return COMMAND_RESULT_ERROR, COMMAND_RESULT_DISCONNECT or COMMAND_RESULT_OK
                return COMMAND_RESULT_OK;
            }
            else{
                SimpleServerSend(Client, "::LastImportFailed");
                // FIXME: Should we return COMMAND_RESULT_ERROR, COMMAND_RESULT_DISCONNECT or COMMAND_RESULT_OK
                return COMMAND_RESULT_OK;
            }
        }
        else{
            sprintf(StatusTemp, "(%d) Command: Data CRC failed: Client(%x) / Server(%x).", MyServer.ClientID[Client], ClientFileCRC, CI[Client].DataCRC);
            WriteLog(StatusTemp);
            SimpleServerSend(Client, "::DataCRCFailed");
            // FIXME: Should we return COMMAND_RESULT_ERROR, COMMAND_RESULT_DISCONNECT or COMMAND_RESULT_OK
            return COMMAND_RESULT_OK;
        }

        WriteLog("Error in ProcessCommand :: Logic failed in FileCRC.");
        return COMMAND_RESULT_ERROR; // Should never get here
    }
    else if((FlagLength = CompareFlagedString(ReceivedData, "::FileSize:")))
    {
        sprintf(StatusTemp, "(%d) Received Command: FileSize.", MyServer.ClientID[Client]);
        WriteLog(StatusTemp);

    	FlaggedLength = GetFlagedValue(ReceivedData, FlagLength, ReceivedSize, FlaggedData);
        FlaggedData[FlaggedLength] = '\0';

        CI[Client].FileSize = atol(FlaggedData);

        sprintf(StatusTemp, "(%d) Command: FileSize: %d.", MyServer.ClientID[Client], CI[Client].FileSize);
        WriteLog(StatusTemp);

        return COMMAND_RESULT_OK;
    }

    sprintf(StatusTemp, "(%d) Received Command: Unknown command.", MyServer.ClientID[Client]);
    WriteLog(StatusTemp);

    return COMMAND_RESULT_UNKNOWN;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

