////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

struct _SQLImportMembers{
    char **ColumbNames;
    char **SingleRowData;
};

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeSingleRowDataColumbs(struct _SQLImportMembers *SIM, int ColumnCount)
{
    int ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        free(SIM->SingleRowData[ColumbPos]);
        SIM->SingleRowData[ColumbPos] = NULL;
        ColumbPos++;
    }
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

void FreeSingleRowData(struct _SQLImportMembers *SIM)
{
    free(SIM->SingleRowData);
    SIM->SingleRowData = NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

bool ImportSQLResults(int Client, char *InFileName)
{
    char StatusText[255];

    FILE *SourceHandle = NULL;

    _SQLImportMembers SIM;

    SQLBasics SQL;

    HSTMT StatementHandle;

    sprintf(StatusText, "(%d) Importing client data.", MyServer.ClientID[Client]);
    WriteLog(StatusText);

    if(!SQL.DBConnect("{SQL Server}", "NetBase", "sa", "SQL840823110", CI[Client].CompanyDB))
    {
        // FIXME: Need to dump the connection variables that failed to log file
        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: DBConnect error.", MyServer.ClientID[Client]);
        WriteLog(StatusText);
        return false;
    }

    if((SourceHandle = fopen(InFileName, "rb")) == NULL)
    {
        // FIXME: Need to dump the file name to log

        sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: TargetFile open error.", MyServer.ClientID[Client]);
        WriteLog(StatusText);
        SQL.DBDisconnect();
        return false;
    }

    int TotalLengthOfAllColumbNames = 0; // For better memory management;
    int TotalLengthOfAllRowData = 0; // For better memory management
    int ImportTableNameLen = 0;
    SQLSMALLINT ColumnCount = 0;
    SQLINTEGER RowCount = 0;
    char ImportTable[256];

    fread(&ImportTableNameLen, sizeof(ImportTableNameLen), 1, SourceHandle); // Read import table name len
    fread(ImportTable, sizeof(char), ImportTableNameLen, SourceHandle); // Read import table name
    fread(&ColumnCount, sizeof(ColumnCount), 1, SourceHandle); // Read number of columbs
    fread(&RowCount, sizeof(RowCount), 1, SourceHandle);       // Read number of rows

    ImportTable[ImportTableNameLen] = '\0';

    char UniuqeKeyName[5][64]; // Contains the names of the uniuqe columbs
    int UniuqeKeyCol[5];       // Contains the columb position of the uniuqe columbs
    int UniuqeKeysNeeded = 2;  // The ammount of uniuqe keys we have to find
    int UniuqeKeysFound = 0;   // The ammount of uniuqe keys we have found

    sprintf(UniuqeKeyName[0], "OrdNbr");
    sprintf(UniuqeKeyName[1], "CustID");

    SIM.ColumbNames = (char **) calloc(sizeof(char *), ColumnCount + 1);

    SQLSMALLINT ColNameLen = 0;

    int ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        fread(&ColNameLen, sizeof(ColNameLen), 1, SourceHandle);
        SIM.ColumbNames[ColumbPos] = (char *) calloc(sizeof(char), ColNameLen + 1);
        fread(SIM.ColumbNames[ColumbPos], sizeof(char), ColNameLen, SourceHandle);
        SIM.ColumbNames[ColumbPos][ColNameLen] = '\0';
        TotalLengthOfAllColumbNames = (TotalLengthOfAllColumbNames + ColNameLen);

        // Is this columb one of the uniuqe keys
        if(UniuqeKeysFound < UniuqeKeysNeeded)
        {
            int UniuqeKeyCount = 0;
            while(UniuqeKeyCount < UniuqeKeysNeeded)
            {
                if(strcmpi(UniuqeKeyName[UniuqeKeyCount], SIM.ColumbNames[ColumbPos]) == 0)
                {
                    UniuqeKeyCol[UniuqeKeyCount] = ColumbPos;
                    UniuqeKeysFound++;
                }
                UniuqeKeyCount++;
            }
        }

        ColumbPos++;
    }
/*
    ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        free(SIM.ColumbNames[ColumbPos]);
        SIM.ColumbNames[ColumbPos] = NULL;
        ColumbPos++;
    }
    free(SIM.ColumbNames);
    SIM.ColumbNames = NULL;
*/
/* // Debug code
    ColumbPos = 0;
    while(ColumbPos < ColumnCount)
    {
        printf("SIM.ColumbNames[%d]:%s\n", ColumbPos, SIM.ColumbNames[ColumbPos]);
        ColumbPos++;
    }

    printf("UniuqeKeysFound: %d\n", UniuqeKeysFound);

	int UniuqeKeyCount = 0;
    while(UniuqeKeyCount < UniuqeKeysFound)
    {
        printf("UniuqeKeyCol[%d]:%d\n", UniuqeKeyCount, UniuqeKeyCol[UniuqeKeyCount]);
        UniuqeKeyCount++;
    }
*/

    sprintf(StatusText, "(%d) Starting row by row data import.", MyServer.ClientID[Client]);
    WriteLog(StatusText);

    SIM.SingleRowData = (char **) calloc(sizeof(char *), ColumnCount + 1);

    SQLINTEGER RowDataLen = 0;

    int RowPos = 0;
    while(RowPos < RowCount)
    {
        int BaseMemoryRequirement = 0;
        TotalLengthOfAllRowData = 0;

        // Get row data
        ColumbPos = 0;
        while(ColumbPos < ColumnCount)
        {
            fread(&RowDataLen, sizeof(RowDataLen), 1, SourceHandle);
            SIM.SingleRowData[ColumbPos] = (char *) calloc(sizeof(char), RowDataLen + 1);
            fread(SIM.SingleRowData[ColumbPos], sizeof(char), RowDataLen, SourceHandle);
            SIM.SingleRowData[ColumbPos][RowDataLen] = '\0';
            TotalLengthOfAllRowData = (TotalLengthOfAllRowData + RowDataLen);
            ColumbPos++;
        }

        BaseMemoryRequirement = ((TotalLengthOfAllColumbNames + TotalLengthOfAllRowData) + (ColumnCount * 5)) + 1024;

        // Work with row data here

//        ColumbPos = 0;
//        while(ColumbPos < ColumnCount)
//        {
//            printf("%s, ", SIM.SingleRowData[ColumbPos]);
//            ColumbPos++;
//        }


        char WhereStatement[1024];
        char CheckExistanceStatement[1024];

        // Assemble the where statement from the row data, the uniuqe key names and positions
        if(UniuqeKeysFound > 0)
        {
            // Assemble the where statement
            sprintf(WhereStatement, "WHERE %s = %s", UniuqeKeyName[0], SIM.SingleRowData[UniuqeKeyCol[0]]);
        	int UniuqeKeyCount = 1;
            while(UniuqeKeyCount < UniuqeKeysFound)
            {
                strcat(WhereStatement, " AND ");
                strcat(WhereStatement, UniuqeKeyName[UniuqeKeyCount]);
                strcat(WhereStatement, " = ");
                strcat(WhereStatement, SIM.SingleRowData[UniuqeKeyCol[UniuqeKeyCount]]);
                UniuqeKeyCount++;
            }

            // Assemble the check existance statement
            sprintf(CheckExistanceStatement, "SELECT %s", UniuqeKeyName[0]);
            strcat(CheckExistanceStatement, " FROM ");
            strcat(CheckExistanceStatement, ImportTable);
            strcat(CheckExistanceStatement, " ");
            strcat(CheckExistanceStatement, WhereStatement);

            //printf("CheckExistanceStatement: %s\n", CheckExistanceStatement);
            //printf("WhereStatement: %s\n", WhereStatement);
        }

        if(!SQL.DBExecute(CheckExistanceStatement, &StatementHandle))
        {
            // FIXME: Need to dump the statement that failed to a seperate log file

            sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: CheckExistanceStatement failed to execute.", MyServer.ClientID[Client]);
            WriteLog(StatusText);
            SQL.DBDisconnect();

            FreeSingleRowDataColumbs(&SIM, ColumnCount);
            FreeSingleRowData(&SIM);

            if(SourceHandle) fclose(SourceHandle);
            return false;
        }

        SQLINTEGER ReturnedRowCount = 0;
        if(SQLRowCount(StatementHandle, &ReturnedRowCount) != SQL_SUCCESS)
        {
            // FIXME: Need to dump the statement that returned these rows seperate log file. (CheckExistanceStatement)

            sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: Failed to retrieve row count.", MyServer.ClientID[Client]);
            WriteLog(StatusText);
            SQL.DBCloseCursor(&StatementHandle);
            SQL.DBDisconnect();

            FreeSingleRowDataColumbs(&SIM, ColumnCount);
            FreeSingleRowData(&SIM);

            if(SourceHandle) fclose(SourceHandle);
            return false;
        }

        SQL.DBCloseCursor(&StatementHandle);

        char *ImportStatement = NULL;

        //----------------------------------------------------------------------------------------------
        if(ReturnedRowCount == 0) // Do insert
        {
            ImportStatement = (char *) calloc( sizeof(char), BaseMemoryRequirement + 1);

//            printf("Inserting row %d. Memory: %d\n", RowPos, BaseMemoryRequirement);

            sprintf(ImportStatement, "INSERT INTO %s (", ImportTable);
    
            ColumbPos = 0;
            while(ColumbPos < ColumnCount)
            {
                strcat(ImportStatement, SIM.ColumbNames[ColumbPos]);
                if(ColumbPos < ColumnCount - 1)
                {
                    strcat(ImportStatement, ",");
                }
                ColumbPos++;
            }
            strcat(ImportStatement, ") VALUES(");

            ColumbPos = 0;
            while(ColumbPos < ColumnCount)
            {
                strcat(ImportStatement, SIM.SingleRowData[ColumbPos]);
                if(ColumbPos < ColumnCount - 1)
                {
                    strcat(ImportStatement, ",");
                }
                ColumbPos++;
            }
            strcat(ImportStatement, ") ");

//          ErrorMessage(NULL, ImportStatement); // Debug code
        }
        //----------------------------------------------------------------------------------------------
        else { // Do update
            BaseMemoryRequirement = (BaseMemoryRequirement + strlen(WhereStatement));
            ImportStatement = (char *) calloc( sizeof(char), BaseMemoryRequirement + 1);

//            printf("Updating row %d. Memory: %d\n", RowPos, BaseMemoryRequirement);

            sprintf(ImportStatement, "UPDATE %s SET ", ImportTable);

            ColumbPos = 0;
            while(ColumbPos < ColumnCount)
            {
                strcat(ImportStatement, SIM.ColumbNames[ColumbPos]);
                strcat(ImportStatement, "=");
                strcat(ImportStatement, SIM.SingleRowData[ColumbPos]);
                if(ColumbPos < ColumnCount - 1)
                {
                    strcat(ImportStatement, ",");
                }
                ColumbPos++;
            }

            strcat(ImportStatement, " ");
            strcat(ImportStatement, WhereStatement);

//            ErrorMessage(NULL, ImportStatement); // Debug code
        }

        //----------------------------------------------------------------------------------------------

        if(ImportStatement != NULL)
        {
//            ErrorMessage(NULL, "Importing"); // Debug code

            if(!SQL.DBExecute(ImportStatement, &StatementHandle))
            {
                // FIXME: Need to dump the statement that failed to a seperate log file. (ImportStatement)

                sprintf(StatusText, "(%d) Error in ImportSQLResultsFailed:: ImportStatement failed to execute.", MyServer.ClientID[Client]);
                WriteLog(StatusText);

                WriteLog(ImportStatement); // Debug Code
                SQL.DBDisconnect();

                FreeSingleRowDataColumbs(&SIM, ColumnCount);
                FreeSingleRowData(&SIM);

                free(ImportStatement);
                ImportStatement = NULL;

                if(SourceHandle) fclose(SourceHandle);
                return false;
            }

            SQL.DBCloseCursor(&StatementHandle);

            free(ImportStatement);
            ImportStatement = NULL;
        }

        ImportStatement = NULL;

        FreeSingleRowDataColumbs(&SIM, ColumnCount);
        //FreeSingleRowData(&SIM); // This memory needs to stay intact untill we are done with the import

        ColumbPos++;

        RowPos++;
    }

    sprintf(StatusText, "(%d) Successfully imported %d client data rows.", MyServer.ClientID[Client], RowPos);
    WriteLog(StatusText);

    FreeSingleRowData(&SIM);

    SQL.DBDisconnect();
    if(SourceHandle) fclose(SourceHandle);

    return true;
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

